#! perl
# Write the contents each of file in the directory (arg 1)
# to standard output, with an appropriate title banner

use strict;
use DirHandle;
use Cwd;
use File::Basename;

# Output formatting
sub bumpIndent {
	return $_[0].'    ';
	}

# XML Strip tools
sub isXml {
	return ($_[0] =~ m!^\s*//<!); 	# uses slash (/) in pattern
	}

sub getXml {
	my ( $strLine, $inpSrc, $strIndent ) = @_;

	my ( $strXml ) = $strLine;
	chomp $strXml;

	# Extract XML-Comment body
	$strXml =~ s!^\s*//<(.*)$!\1!;		# uses slash (/) in pattern

	# Extract the context operator
	my ( $cOp ) = substr($strXml, 0, 1);
	$strXml =~ s!^.\s*(.*)$!\1!;		# strip "<" leading spaces

	# Extract the tag name
	my ( $strTag ) = $strXml;
	$strTag =~ s!^(\S+).*!\1!;

	#$ LATER:  Handle continued XML lines

	# Use context operator to grab Java content
	my ( $strContent );

	# "+": Append next line and leave element open
	if ($cOp eq "+") {
		$strContent = <$inpSrc>;
		$strContent =~ s!^\s*(.*)$!\1!;	# strip content leading spaces
		chomp($strContent);

		$strIndent = bumpIndent($strIndent);
		$strXml .= "\n$strIndent$strContent";
		}

	# "=": Append next line and close tag
	elsif ($cOp eq "=") {
		$strContent = <$inpSrc>;
		$strContent =~ s!^\s*(.*)$!\1!;	# strip content leading spaces
		chomp($strContent);

		$strIndent = bumpIndent($strIndent);
		$strXml .= "\n$strIndent$strContent\n";
		$strXml .= "$strIndent</$strTag>";
		}

	# "/": No context, end tag only
	elsif ($cOp eq "/") {
		$strXml = "/$strXml";
		}

	# "-": No context, empty tag only
	elsif ($cOp eq "-") {
		# nothing; leave $strXml unchanged
		}

	# Something borrowed?  Something new?
	else {
		print STDERR "Unknown context operator '$cOp' on line:\n\t$strLine\n";
		}

	return '<'.$strXml;
	}

sub stripXmlFile {
	my ( $inpSrc, $outDst, $strIndent ) = @_;
	while (<$inpSrc>) {
		if (isXml($_)) {
			my $strXml = getXml($_, $inpSrc, $strIndent);
			print $outDst $strIndent, $strXml, "\n";
			}
		}
	}

sub stripJavaXml {
	my ( $strSrc, $outDst, $strIndent ) = @_;
	local *SRC;

	open SRC, $strSrc;
	stripXmlFile \*SRC, $outDst, $strIndent;
	}


# Include embedded XML  files
sub copyXmlFile {
	my ( $strSrc, $outDst, $strIndent ) = @_;
	local *SRC;

	open SRC, $strSrc;
	while (<SRC>) {
		chomp;
		next if (m/<?xml /);
		print $outDst $strIndent, $_, "\n";
		}
	}

sub includeXmlFile {
	my ( $strPath, $strFile, $outDst, $strIndent ) = @_;

	# Include XML file if it exists
	my ($strTest) = "$strPath/$strFile";
	if (-e $strTest) {
		copyXmlFile $strTest, $outDst, $strIndent;
		}
	}

# Support JAVA files
sub isJava {
	return ( $_[0] =~ m/.java$/ );
	}

sub doJavaWork {
	my ($strLabel, $strSource, $outDst, $strIndent) = @_;

	print $outDst "$strIndent<file name=\"$strLabel\">\n";
	$strIndent = bumpIndent($strIndent);

	stripJavaXml $strSource, $outDst, $strIndent;

	print $outDst $strIndent."</file>\n";
    	}

sub doFileWork {
	my ($strLabel, $strSource, $outDst, $strIndent) = @_;

	if (isJava($strSource)) {
		doJavaWork $strLabel, $strSource, $outDst, $strIndent;
		}
	}

# Directory recursion
sub enumDir {
    my ( $strDir ) = @_;
    my @strResult;
    my $strFile;
    my $dirHere;

    $dirHere = new DirHandle $strDir;
    if (defined $dirHere) {

        while (defined($strFile = $dirHere->read)) {
            if ('.' eq $strFile) {}
            elsif ('..' eq $strFile ) {}
            else {
                push @strResult, $strFile;
                }
            }
        }

    return @strResult
    }

sub makePath {
    my $strResult = '';

    for my $strTerm (@_) {
	next if ('' eq $strTerm);

        $strResult .= '/' if ('' ne $strResult);
        $strResult .= $strTerm;
        }

    return $strResult;
    }

sub doDirWork {
    my ($strRoot, $strFull, $outDst, $strIndent) = @_;
    my @strList = enumDir($strFull);

    my @strSub = ();

    # Try to print each real file
    foreach my $strFile (@strList) {

	# Avoid CVS directories
	next if ($strFile eq 'CVS');

	my $strNewFull = $strFull.'/'.$strFile;
        my $strNewRoot = makePath($strRoot, $strFile);

	# Standard file validation
	if (-d $strNewFull) {
            push @strSub, $strFile;
            }
	else {
            doFileWork($strNewRoot, $strNewFull, $outDst, $strIndent);
            }
        }

    # Recurse on each directory
    foreach my $strChild (@strSub) {
	my $strNewFull = makePath($strFull, $strChild);
	my $strNewRoot = makePath($strRoot, $strChild);
        stripTreeXml($strChild, $strNewRoot, $strNewFull, $outDst, $strIndent);
        }
    }

sub stripTreeXml {
	my ($strName, $strRoot, $strFull, $outDst, $strIndent) = @_;

	# Begin the directory
	print $outDst "$strIndent<dir name=\"$strFull\">\n";
	$strIndent = bumpIndent($strIndent);
	print $outDst $strIndent."<name>$strName</name>\n";

	# Include dimension data first
	includeXmlFile $strFull, 'Dimension.xml', $outDst, $strIndent;

	# Import the rest of the tree
	doDirWork $strFull, $strRoot, $outDst, $strIndent;

	# Close the directory
	print $outDst $strIndent."</dir>\n";
	}


sub stripUniverse {
	my ( $strRoot, $strFull, $outDst, $strIndent ) = @_;

	print $outDst "<?xml version=\"1.0\"?>\n";

	$strIndent = bumpIndent($strIndent);
	my ( $strNsIndent ) = bumpIndent($strIndent);
	print $outDst "<universe>\n";
	stripTreeXml basename($strFull), $strRoot, $strFull, $outDst, $strIndent;
	print $outDst $strIndent."</universe>\n";
	}


stripUniverse($ARGV[0], $ARGV[0], \*STDOUT, '');
